! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!*****************************************************************
!
!
!****************************************************************
module statistics_mod
    use param
    use cal
    use dal
 
  use oal
  use caloaldefs
  use graph_mod  

  
  use errorHandling
    use sasdal
  
    use omnumerics_mod 
    use omallocation_mod
   



contains

!*****************************************************
! Subroutine DrawNormalDistribution
!*****************************************************
subroutine DrawNormalDistribution(noData, binWidth, mean, standardDeviation, &
          colour, disp, lineStyle)
    integer, intent(in) :: noData
    real(kind=double), intent(in) :: mean, standardDeviation
    integer, intent(in), optional :: colour, disp, lineStyle
    real(kind=double) :: variance
    real(kind=single), allocatable, dimension(:) :: xData, yData
    integer i, disp1
    real(kind=double) xValue, yValue, dx
    real xmin, xmax, ymin, ymax
    integer, parameter :: size = 100
    real(kind=double), intent(in) ::  binWidth
    real(kind=double) ::  constant
    real(kind = double), parameter :: PI = 3.141592653589793_double
    allocate(xData(1 : size))
    allocate(yData(1 : size))

    variance = standardDeviation * standardDeviation
    constant = dble(noData) * binWidth / (standardDeviation * sqrt(2.0_double * PI))

    dx = (xmax - xmin) / real(size - 1)

    do i = 1, size

        xValue = xmin + dx * real(i  - 1)
        yValue = constant * Gaussian(xValue, mean, variance)
        xData(i) = xValue         
        yData(i) = yValue

    end do
    if(present(colour)) then
    else    
    end if
    if(present(lineStyle)) then
    else    
    end if
    call drawline(size, xData, yData)
    deallocate(xData)
    deallocate(yData)


    if(present(disp)) then
        disp1 = -disp
    else
        disp1 = -4
    end if






end subroutine DrawNormalDistribution



!*******************************************
!
! Function Gaussian
!
!*******************************************
real(kind=double) function Gaussian(x, mean, variance)
    implicit none
    real(kind=double), intent(in) :: x, mean, variance
    
    Gaussian = exp(-((x - mean) ** 2) / &
               (2.0_double * variance))
end function Gaussian






!*****************************************************
!
! Subroutine DrawPoissonDistribution
!
!*****************************************************
subroutine DrawPoissonDistribution(noData, binWidth, mean, &
             colour, disp, lineStyle)
    integer, intent(in) :: noData
    real(kind=double), intent(in) :: binWidth, mean
    integer, intent(in), optional :: colour, disp, lineStyle
    real(kind=single), allocatable, dimension(:) :: xData, yData
    integer i, disp1
    real(kind=double) :: yValue
    integer :: xValue, size = 100
    real xmin, xmax, ymin, ymax
  
    
    real(double) ::  constant
  
    character(100) :: text
 
 
     
    constant = dble(noData) * binWidth

    xmin = max(0, floor(xmin))
    xmax = ceiling(xmax)
    size = int(xmax - xmin + 0.01)


    allocate(xData(1 : size))
    allocate(yData(1 : size))
 



!write(*, *) "DRAWPOISSONDISTRIBUTION"
!write(*, *) "nodata=", noData, " binwdith=", binwidth, " mean=", mean
!write(*, *) "constant=", constant

    do i = 1, size

        xValue = int(xmin + 0.00001)  + (i  - 1)
        yValue = constant * Poisson(xValue, mean)
!write(*, *) i, ") x=", xvalue, " y=", yvalue

        xData(i) = xValue         
        yData(i) = yValue

    end do





    if(present(colour)) then
    else
    end if

    if(present(disp)) then
        disp1 = -disp
    else
        disp1 = -4
    end if

  

    if(present(lineStyle)) then
    else    
    end if



    call drawline(size, xData, yData)

    deallocate(xData)
    deallocate(yData)


   

    
    write(text, '(a, f0.1)' )  "POISSON WITH MEAN ", mean






end subroutine DrawPoissonDistribution
!**************************************************************
!
!
!
!***********************************************************
real(double) function Poisson(x, mean)
    integer, intent(in) :: x
    real(double), intent(in) :: mean
   
    real(double) :: p0, p1, p2
    integer :: i

    

 
    if( x < 0) then
        Poisson = -1.0_double

    end if


 
    p0 = exp(-mean) 
    if( x == 0) then
        Poisson = p0
        return

    end if  

    p1 = p0

    do i = 1, x
        p2 = (mean / dble( i)) * p1
      
 
        p1 = p2

    end do

    
  
    
    Poisson = p2 

!    write(*, *) "mean=", mean, " x=", x, " poisson=", p2

end function Poisson

!*******************************************************************
!
!
!*****************************************************************
subroutine poissontest(nData, data, mean, variance, ratio)
     integer, intent(in) :: nData
     real(single) :: data(:)
     real(double), intent(out) :: mean, variance, ratio
     real(double) :: std

     call ComputeAverage4( dble(data), nData, mean, std)
     variance = std * std

     if(mean  .gt. 0.0) then
         ratio = variance / mean
     else
        ratio = 0.0_double
     end if

    

end subroutine poissonTest




!**********************************************************************
!
!
!*********************************************************************
real(single) function Percentile(nData, data, frac)
    integer, intent(in) :: ndata
    real(single) :: data(:)
    real(single), intent(in) :: frac

    real(single), pointer :: temp(:)=>null()
    integer :: k

    if(ndata .eq. 0 ) then
        Percentile = 0.0
        return

    end if

    temp => allocateSingle(1, nData, "temp")
    temp = data(1 : ndata)


    call heapSort(temp, nData)

    k = int(frac * real(ndata) + 0.00001)
    k = max(1, k)

    Percentile = temp(k)
    deallocate(temp)
    nullify(temp)
    

end function percentile    












end module statistics_mod


























